let utils = {};

//function to send ajax call with promise
utils.get = (url) => {
  return new Promise(function(resolve,reject){
  var req = new XMLHttpRequest();
  req.open('GET',url);
  req.onload = function() {
    // success
    if (req.status == 200)  {
        resolve(req.response);
    }
    else {
      reject(Error('promise error with ' + req.status));
    }
  };
  req.onerror = function(err) {
    reject(Error('Network Error with '+url+': ' + err));
  };
  req.send();
  });
}

utils.getJSON = async function(url) {
  
  var data = {};
  
  var string = null;
  try {
    string = await utils.get(url);
  }
  catch (e) {
    alert('error: ' + e);
  }
  try { 
      data = JSON.parse(string);
      success = true;
  }
  catch (e) {
     alert('parse error');
  }
  return data;
}

//init function to get the quiz json
async function init() {
  let root = document.querySelector('#root');
  let url = 'https://proto.io/en/jobs/candidate-exercise/quiz.json';
  let quiz= null;
  try {
     quiz = await utils.getJSON(url);
  }
  catch (e) {
     root.textContent = 'error: ' + e;
  }
  buildInitialHtml(quiz);
}

//build initial htlm for question 1
function buildInitialHtml(quiz) {
  
  //save questions to local storage
  localStorage.setItem('questions', JSON.stringify(quiz.questions));
  localStorage.setItem('numberOfQuestions', JSON.stringify(quiz.questions.length));

  //add points variable to session
  sessionStorage.setItem('points',0);

  //add quiz_id to session
  sessionStorage.setItem('quiz_id',quiz.quiz_id);

  //save maximum possible points to session
  let max_points = 0;
  quiz.questions.forEach(function (question, index) {
    max_points = +max_points + +question.points; 
  });
  sessionStorage.setItem('maxPoints',max_points);

  //add description of quiz to html
  let description = document.querySelector('#description');  
  description.innerHTML = quiz.description;
  
  //add title of quiz to title tag
  let title = document.querySelector('#title'); 
  title.innerHTML = quiz.title + ' - ' + quiz.quiz_id;

  buildQuestion(1);
  
}

//function to retrieve question based on index 
function buildQuestion(index) {

    //get questions from local storage
    let questions = JSON.parse(localStorage.getItem('questions'));
    var question = questions[index-1];
    var question_id = question.q_id;

    //create the progress html block
    var progress = document.querySelector('#progressBar');
    progress.innerHTML = 'Question ' + index + ' of ' + questions.length;

    //question's html block
    var html = '<p><img src='+question.img+' />';
    html += '<p>' + question.title + '</p>';
    html += '<div class="answers" data-id="'+question_id+'" data-type="'+question.question_type+'">';

    //based on question type construct answers
    switch(question.question_type) {
      case 'multiplechoice-single':
        var possible_answers = question.possible_answers;
        html += single(possible_answers,question_id);
        break;
      case 'truefalse':
        html += truefalse(question_id);
        break;
      case 'multiplechoice-multiple':
        var possible_answers = question.possible_answers;
        html += multiple(possible_answers,question_id);
        break;
      default:
        console.log('This type of question is not supported');
    }

    html += '</div>';
    
    let question_html = document.querySelector('#question');
    question_html.innerHTML = html;
}

//construct single multiplechoice question
function single(possible_answers,question_id) {

  var html = '';

  possible_answers.forEach(function (answer, index) {
    html += '<div class="form-check">';
    html += '<input type="radio" name="'+question_id+'" class="form-check-input" value="'+answer.a_id+'">';
    html += '<label class="form-check-label" for="'+answer.a_id+'">'+answer.caption+'</label>';
    html += '</div>';
  });

  return html;
}

//construct multiple multiplechoice question
function multiple(possible_answers,question_id) {

  var html = '';

  possible_answers.forEach(function (answer, index) {
    html += '<div class="form-check">';
    html += '<input type="checkbox" class="form-check-input" name="'+question_id+'" value="'+answer.a_id+'">';
    html += '<label class="form-check-label" for="'+answer.a_id+'">'+answer.caption+'</label>';
    html += '</div>';
  });
  
  return html;
}

//construct true or false question
function truefalse(question_id) {

    var html = '';

    html += '<div class="form-check">'; 
    html += '<input type="radio" class="form-check-input" name="'+question_id+'" value="true">';
    html += '<label class="form-check-label" for="True">True</label>';
    html += '</div>';
    html += '<div class="form-check">'; 
    html += '<input type="radio" class="form-check-input" name="'+question_id+'" value="false">';
    html += '<label class="form-check-label" for="False">False</label>';
    html += '</div>';

    return html;

}

//validate function.
function validate(question) {
  
  var userAnswers = findSelection(question); 

  //if user did not choose an answer alert and stay on the same question
  if (userAnswers.length > 0 ) {
    var questions = JSON.parse(localStorage.getItem('questions'));
    var question = questions[question-1];
    var correctAnswer = question.correct_answer;
    var points = sessionStorage.getItem('points');

    //if user's answer is correct append the points
    if (new String(userAnswers).valueOf() === new String(correctAnswer).valueOf()) {
      points = +points + +question.points;
      sessionStorage.setItem('points',points);
    }

    //highlight correct answers
    var inputs = document.getElementsByClassName('form-check-input');
    
    if(Array.isArray(correctAnswer)) {
      correctAnswer.forEach(function (result, index) {
          for (i=0;i < inputs.length; i++) {
            if (new String(inputs[i].value).valueOf() === new String(result).valueOf() ) {
              inputs[i].classList.add('green');
              var labelElement = inputs[i].nextElementSibling;
              labelElement.classList.add('green');
            }
          }
      });
    } else {
      for (i=0;i < inputs.length; i++) {
            if (new String(inputs[i].value).valueOf() === new String(correctAnswer).valueOf() ) {
              inputs[i].classList.add('green');
              var labelElement = inputs[i].nextElementSibling;
              labelElement.classList.add('green');
            }
          }
    }
    
    return true;
  } else {
      alert('Please select at least one answer');
      return false;
  }
}

init();

document.querySelector('#next')
 .addEventListener('click', (e) => {

    let question_id = document.querySelector(".answers");
    let currentQuestion = question_id.getAttribute('data-id');
    let nextQuestion = +currentQuestion + +1;
    let numberOfQuestions = JSON.parse(localStorage.getItem('numberOfQuestions'));

    //validate
    if(validate(currentQuestion)) {
      setTimeout(function () {
          if (currentQuestion == numberOfQuestions) {
            initResults();
          } else {
            buildQuestion(nextQuestion);
          }
      }, 3000);
    }
});

 function findSelection(field) {
    let radios = document.getElementsByName(field);
    let answers = '';

    for (var i = 0, length = radios.length; i < length; i++) {
      if (radios[i].checked) {
        // do whatever you want with the checked radio
        if(answers != '' ) {
          answers += ',' + radios[i].value;
        } else {
          answers += radios[i].value;
        }
      }
    }
    return answers;
}


async function initResults() {

  let root = document.querySelector('#root');
  root.innerHTML = 'Thank you';

  let url = 'https://proto.io/en/jobs/candidate-exercise/result.json';
  let result= null;
  try {
     result = await utils.getJSON(url);
  }
  catch (e) {
     root.textContent = 'error: ' + e;
  }
  buildResult(result);
}

function buildResult(results) {
  
  //Get quiz id stored in session from quiz.json
  let quiz_id = sessionStorage.getItem("quiz_id");

  //Get userPoints and maxPoints to make the calculations in %
  let userPoints = sessionStorage.getItem("points");
  let maxPoints = sessionStorage.getItem("maxPoints");
  let points = (userPoints*100) / maxPoints;

  //initialize html to replace with results
  let html = '';

  //if quiz_id saved from quiz.json is same as results.json then proceed
  if (quiz_id == results.quiz_id) {
      //loop through results and find appropriate result information according to points acquired by the user
      results.results.forEach(function (result, index) {
        let minpoints = result.minpoints;
        let maxpoints = result.maxpoints;
        if ( points >= minpoints && points <= maxpoints ) {
          html = '<p><img src='+result.img+' />';
          html += '<p>Your score is '+ points +'/100</p>';
          html += '<p>' + result.title + '</p>';
          html += '<p>'+ result.message +'</p>';
        }
      });

      let root = document.querySelector('#root');
      root.innerHTML = html;
  }
  else {
    alert("Something went wrong.");
  }


}